// BOOSTCA.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <iostream>

#include <boost/asio/io_service.hpp>
#include <boost/scope_exit.hpp>
#include <boost/thread.hpp>
#include <boost/asio.hpp>
#include <boost/bind.hpp>

using namespace std;

std::vector<unsigned char> message(256, 42);

boost::asio::io_service ioservice;

void PipeWriteMessage(HANDLE serverHandle)
{
   boost::asio::windows::stream_handle serverStream(ioservice, serverHandle);

   cout << "Sending message..." << endl;

   serverStream.write_some(boost::asio::buffer(message));
}

int main(int argc, char* argv[])
{
   const string pipeName = "\\\\.\\pipe\\MyPipeServ";

   //create pipe serv
   HANDLE serverHandle = CreateNamedPipeA(
      pipeName.c_str(), // pipe name
      PIPE_ACCESS_DUPLEX | // read/write access
      FILE_FLAG_OVERLAPPED, // overlapped mode
      PIPE_TYPE_MESSAGE | // message-type pipe
      PIPE_READMODE_MESSAGE | // message-read mode
      PIPE_WAIT, // blocking mode
      1, // number of instances
      message.size(), // output buffer size
      message.size(), // input buffer size
      INFINITE, // client time-out
      NULL); // default security attributes

   //create pipe client
   HANDLE clientHandle = CreateFileA(
      pipeName.c_str(), // pipe name
      GENERIC_READ | // read and write access
      GENERIC_WRITE, 
      0, // no sharing 
      NULL, // default security attributes
      OPEN_EXISTING, // opens existing pipe
      FILE_FLAG_OVERLAPPED, // default attributes
      NULL); // no template file

   // The pipe connected; change to message-read mode. 
   DWORD dwMode = PIPE_READMODE_MESSAGE;
   BOOL fSuccess = SetNamedPipeHandleState(
      clientHandle, // pipe handle 
      &dwMode, // new pipe mode 
      NULL, // don't set maximum bytes 
      NULL); // don't set maximum time

   boost::asio::windows::stream_handle clientStream(ioservice, clientHandle);

   boost::thread pipeWriteMessageThread(boost::bind(PipeWriteMessage, serverHandle));

   BOOST_SCOPE_EXIT(&pipeWriteMessageThread)
   {
      pipeWriteMessageThread.join();
   }
   BOOST_SCOPE_EXIT_END

   size_t bytesRead = 0;

   std::vector<unsigned char> readBuf(message.size() / 2);

   do
   {
      try
      {
         bytesRead += clientStream.read_some(boost::asio::buffer(readBuf));
      }
      catch(boost::system::system_error& err)
      {
         cout << "Error! Error info : " << err.what() << endl;

         if (err.code().value() != ERROR_MORE_DATA)
         {
            throw;
         }
      }
   }
   while(bytesRead < message.size());

   cout << "Success! The whole message was received." << endl;

   return 0;
}
